# coding: utf-8
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import unicode_literals

import re

from testinfra.modules.base import InstanceModule


class PipPackage(InstanceModule):
    """Test pip packages status and version"""

    def get_packages(self, pip_path="pip"):
        """Get all installed packages and versions returned by `pip list`:

        >>> PipPackage.get_packages(pip_path='~/venv/website/bin/pip')
        {'Django': {'version': '1.10.2'},
         'mywebsite': {'version': '1.0a3', 'path': '/srv/website'},
         'psycopg2': {'version': '2.6.2'}}
        """
        output_re = re.compile(r'^(.+) \((.+)\)$')
        pkgs = {}
        out = self.check_output("%s list --no-index", pip_path)
        for line in out.splitlines():
            if line.startswith('Warning: '):
                # Warning: cannot find svn location for rst2pdf==0.93.dev-r0
                continue
            match = output_re.match(line)
            if match is None:
                raise RuntimeError("could not parse {0}".format(repr(line)))
            name, version = match.groups()
            if ',' in version:
                version, path = version.split(',', 1)
                path = path.strip()
                pkgs[name] = {'version': version, 'path': path}
            else:
                pkgs[name] = {'version': version}
        return pkgs

    def get_outdated_packages(self, pip_path="pip"):
        """Get all outdated packages with current and latest version

        >>> PipPackage.get_outdated_packages(pip_path='~/venv/website/bin/pip')
        {'Django': {'current': '1.10.2', 'latest': '1.10.3'}}
        """
        output_re = [re.compile(r) for r in [
            # newer pip
            r'^(.+) \(Current: (.+) Latest: (.+)\)$',
            # old pip
            r'^(.+) \((.+)\) - Latest: (.+) .*$',
        ]]
        pkgs = {}
        out = self.check_output("%s list -o", pip_path)
        for line in out.splitlines():
            # Warning: cannot find svn location for rst2pdf==0.93.dev-r0
            # Could not find any downloads that satisfy the requirement iotop
            if line.startswith('Warning: ') or line.startswith('Could not '):
                continue
            for out_re in output_re:
                match = out_re.match(line)
                if match:
                    name, current, latest = match.groups()
                    pkgs[name] = {'current': current, 'latest': latest}
                    break
            else:
                raise RuntimeError("could not parse {0}".format(repr(line)))
        return pkgs



# coding: utf-8
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals
import pytest
import six

import testinfra.backend
BACKENDS = ("ssh", "safe-ssh", "docker", "paramiko", "ansible")
HOSTS = [backend + "://debian_jessie" for backend in BACKENDS]
USER_HOSTS = [backend + "://user@debian_jessie" for backend in BACKENDS]
SUDO_HOSTS = [
    backend + "://user@debian_jessie?sudo=True"
    for backend in BACKENDS
]
SUDO_USER_HOSTS = [
    backend + "://debian_jessie?sudo=True&sudo_user=user"
    for backend in BACKENDS
]


@pytest.mark.testinfra_hosts(*(
    HOSTS + USER_HOSTS + SUDO_HOSTS + SUDO_USER_HOSTS))
def test_command(Command):
    assert Command.check_output("true") == ""
    # test that quotting is correct
    assert Command("echo a b | grep -q %s", "a c").rc == 1


@pytest.mark.testinfra_hosts(*HOSTS)
def test_encoding(TestinfraBackend, Command):
    if TestinfraBackend.get_connection_type() == "ansible":
        pytest.skip("ansible handle encoding himself")

    # jessie image is fr_FR@ISO-8859-15
    cmd = Command("ls -l %s", "/é")
    if TestinfraBackend.get_connection_type() == "docker":
        # docker bug ?
        assert cmd.stderr_bytes == (
            b"ls: impossible d'acc\xe9der \xe0 /\xef\xbf\xbd: "
            b"Aucun fichier ou dossier de ce type\n"
        )
    else:
        assert cmd.stderr_bytes == (
            b"ls: impossible d'acc\xe9der \xe0 /\xe9: "
            b"Aucun fichier ou dossier de ce type\n"
        )
        assert cmd.stderr == (
            "ls: impossible d'accéder à /é: "
            "Aucun fichier ou dossier de ce type\n"
        )


@pytest.mark.testinfra_hosts(*(USER_HOSTS + SUDO_USER_HOSTS))
def test_user_connection(User):
    assert User().name == "user"


@pytest.mark.testinfra_hosts(*SUDO_HOSTS)
def test_sudo(User):
    assert User().name == "root"


@pytest.mark.testinfra_hosts("ansible://debian_jessie")
def test_ansible_hosts_expand(TestinfraBackend):
    from testinfra.backend.ansible import AnsibleBackend

    def get_hosts(spec):
        return AnsibleBackend.get_hosts(
            spec, ansible_inventory=TestinfraBackend.ansible_inventory)
    assert get_hosts(["all"]) == ["debian_jessie"]
    assert get_hosts(["testgroup"]) == ["debian_jessie"]
    assert get_hosts(["*ia*jess*"]) == ["debian_jessie"]


def test_backend_importables():
    # just check that all declared backend are importable and NAME is set
    # correctly
    for connection_type in testinfra.backend.BACKENDS:
        if six.PY3 and connection_type == 'ansible':
            pytest.skip()
        obj = testinfra.backend.get_backend_class(connection_type)
        assert obj.get_connection_type() == connection_type
